/**
 * Functions for rendering components using Preact "X" (v10 and later) and
 * converting the result to a React Standard Tree (RST) format defined by
 * Enzyme.
 *
 * Preact 10+ stores details of the rendered elements on internal fields of
 * the VNodes. A reference to the vnode is stored in the root DOM element.
 * The rendered result is converted to RST by traversing these vnode references.
 */
import { RSTNode } from 'enzyme';
import { VNode } from 'preact';
declare type RSTNodeTypes = RSTNode | string | null;
/**
 * Convert a JSX element tree returned by Preact's `h` function into an RST
 * node.
 */
export declare function rstNodeFromElement(node: VNode | null | string): RSTNodeTypes;
/**
 * Convert the Preact components rendered into `container` into an RST node.
 */
export declare function getNode(container: HTMLElement): RSTNode;
export {};
