/**
 * Functions for rendering components using Preact "classic" (v9 and below)
 * and converting the result to a React Standard Tree (RST) format defined by
 * Enzyme.
 *
 * Preact <= 8 stores details of the rendered elements on the DOM nodes
 * themselves and updates diff VDOM elements against the DOM. The rendered
 * result is converted to RST by traversing the DOM and Preact-internal
 * metadata attached to DOM nodes.
 */
import { VNode } from 'preact';
import { RSTNode } from 'enzyme';
declare type RSTNodeTypes = RSTNode | string | null;
/**
 * Convert the Preact components rendered into `container` into an RST node.
 */
export declare function getNode(container: HTMLElement): RSTNode;
/**
 * Convert a JSX element tree returned by Preact's `h` function into an RST
 * node.
 */
export declare function rstNodeFromElement(node: VNode | null | string): RSTNodeTypes;
export {};
